package com.persagy.dmp.report.service;

import com.persagy.common.criteria.JpaUtils;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.report.common.json.JsonMapper;
import com.persagy.dmp.report.entity.*;
import com.persagy.dmp.report.enumeration.DateType;
import com.persagy.dmp.report.repository.ReportAlarmRecordRepository;
import com.persagy.dmp.report.repository.ReportObjectRepository;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author fengyanjie
 */
@Slf4j
@Service
public class DataExecutorService {
    @Autowired
    private ReportObjectRepository reportObjectRepository;

    @Autowired
    private ReportAlarmRecordRepository reportAlarmRecordRepository;

    @Autowired
    private ReportUtils reportUtils;

    @Autowired
    private JpaUtils jpaUtils;

    @Transactional
    public void reportRwdObject(DateType dateType, Date startTime, Date endTime) throws Exception {
        String groupCode = DmpParameterStorage.getGroupCode();
        String projectId = DmpParameterStorage.getProjectId();
        String tableName = "rwd_object";
        String sql = "";
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("        select project_id,group_code,obj_type,class_code,'" + dateType + "' as date_type,          \n");
        reportUtils.appendSql(sqlBuilder, dateType);
        sqlBuilder.append("        count(0) rwd_object_num from dmp_rwd." + tableName + "_" + groupCode + "_" + projectId + "          \n");
        sqlBuilder.append("        where create_time >= ? and create_time < ?          \n");
        sqlBuilder.append("          and project_id = ? and group_code = ?          \n");
        sqlBuilder.append("         group by project_id,group_code,obj_type,class_code,date_type,year,month,day,hour,season,date          \n");
        sql = sqlBuilder.toString();
        //TODO 返回字段值为null报错
        List<Map<String, Object>> list = jpaUtils.queryList(sql, startTime, endTime, projectId, groupCode);
        List<BaseEntity> data = new ArrayList<>();
        list.forEach(item -> {
            ReportRwdObject reportRwdObject = new ReportRwdObject();
            String year = MapUtils.getString(item, "year");
            String month = MapUtils.getString(item, "month");
            String day = MapUtils.getString(item, "day");
            String hour = MapUtils.getString(item, "hour");
            String season = MapUtils.getString(item, "season");
            reportRwdObject.setYear(year.equals("null") ? null : year);
            reportRwdObject.setDay(day.equals("null") ? null : day);
            reportRwdObject.setMonth(month.equals("null") ? null : month);
            reportRwdObject.setHour(hour.equals("null") ? null : hour);
            reportRwdObject.setSeason(season.equals("null") ? null : season);
            reportRwdObject.setProjectId(MapUtils.getString(item, "project_id"));
            reportRwdObject.setGroupCode(MapUtils.getString(item, "group_code"));
            reportRwdObject.setObjType(MapUtils.getString(item, "obj_type"));
            reportRwdObject.setClassCode(MapUtils.getString(item, "class_code"));
            reportRwdObject.setCreateTime(MapUtils.getString(item, "date"));
            reportRwdObject.setRwdObjectNum(MapUtils.getLong(item, "rwd_object_num"));
            reportRwdObject.setDateType(DateType.valueOf(MapUtils.getString(item, "date_type")));
            data.add(reportRwdObject);
        });
        ReportConfig reportConfig = reportUtils.queryReportConfig(tableName);
        List<Object> tableGroup = JsonMapper.json2List(JsonMapper.toSimpleJson(reportConfig.getTableGroup()));
        List<ReportConfigGroup> groupList = reportUtils.getGroupList(tableGroup, dateType, startTime, endTime);
        List<BaseEntity> resultList = reportUtils.initData(0, groupList, ReportRwdObject.class, new ArrayList<>());
        List<BaseEntity> result = reportUtils.addList(data, resultList, reportConfig.getCalculatedColumns());

        Map<String, String> map = reportUtils.getDate(dateType, startTime, endTime);
        String start = map.get("start");
        String end = map.get("end");
        reportObjectRepository.deleteDateType(dateType.name(), start, end);
        List<ReportRwdObject> save = new ArrayList<>();
        result.forEach(item -> save.add((ReportRwdObject) item));
        reportObjectRepository.saveAll(save);
    }

    @Transactional
    public void reportAlarmRecord(DateType dateType, Date startTime, Date endTime) throws Exception {
        String groupCode = DmpParameterStorage.getGroupCode();
        String projectId = DmpParameterStorage.getProjectId();
        String tableName = "alarm_record";
        String sql = "";
        StringBuilder sqlBuilder = new StringBuilder();
        sqlBuilder.append("        select item_code,project_id,group_code,category,class_code,level,state,'" + dateType + "' as date_type,          \n");
        reportUtils.appendSql(sqlBuilder, dateType);
        sqlBuilder.append("        count(0) alarm_record_num from dmp_alarm." + tableName + "           \n");
        sqlBuilder.append("        where create_time >= ? and create_time < ?          \n");
        sqlBuilder.append("          and project_id = ? and group_code = ?          \n");
        sqlBuilder.append("         group by item_code,project_id,group_code,category,class_code,level,state,date_type,year,month,day,hour,season,date          \n");
        sql = sqlBuilder.toString();

        List<Map<String, Object>> list = jpaUtils.queryList(sql, startTime, endTime, projectId, groupCode);
        List<BaseEntity> data = new ArrayList<>();
        list.forEach(item -> {
            ReportAlarmRecord reportAlarmRecord = new ReportAlarmRecord();
            String year = MapUtils.getString(item, "year");
            String month = MapUtils.getString(item, "month");
            String day = MapUtils.getString(item, "day");
            String hour = MapUtils.getString(item, "hour");
            String season = MapUtils.getString(item, "season");
            reportAlarmRecord.setYear(year.equals("null") ? null : year);
            reportAlarmRecord.setDay(day.equals("null") ? null : day);
            reportAlarmRecord.setMonth(month.equals("null") ? null : month);
            reportAlarmRecord.setHour(hour.equals("null") ? null : hour);
            reportAlarmRecord.setSeason(season.equals("null") ? null : season);
            reportAlarmRecord.setItemCode(MapUtils.getString(item, "item_code"));
            reportAlarmRecord.setProjectId(MapUtils.getString(item, "project_id"));
            reportAlarmRecord.setGroupCode(MapUtils.getString(item, "group_code"));
            reportAlarmRecord.setCategory(MapUtils.getString(item, "category"));
            reportAlarmRecord.setClassCode(MapUtils.getString(item, "class_code"));
            reportAlarmRecord.setLevel(MapUtils.getString(item, "level"));
            reportAlarmRecord.setAlarmState(MapUtils.getString(item, "state"));
            reportAlarmRecord.setCreateTime(MapUtils.getString(item, "date"));
            reportAlarmRecord.setAlarmRecordNum(MapUtils.getLong(item, "alarm_record_num"));
            reportAlarmRecord.setDateType(DateType.valueOf(MapUtils.getString(item, "date_type")));
            data.add(reportAlarmRecord);
        });
        ReportConfig reportConfig = reportUtils.queryReportConfig(tableName);
        List<Object> tableGroup = JsonMapper.json2List(JsonMapper.toSimpleJson(reportConfig.getTableGroup()));
        List<ReportConfigGroup> groupList = reportUtils.getGroupList(tableGroup, dateType, startTime, endTime);
        List<BaseEntity> resultList = reportUtils.initData(0, groupList, ReportAlarmRecord.class, new ArrayList<>());
        List<BaseEntity> result = reportUtils.addList(data, resultList, reportConfig.getCalculatedColumns());

        Map<String, String> map = reportUtils.getDate(dateType, startTime, endTime);
        String start = map.get("start");
        String end = map.get("end");
        reportAlarmRecordRepository.deleteDateType(dateType.name(), start, end);
        List<ReportAlarmRecord> save = new ArrayList<>();
        result.forEach(item -> save.add((ReportAlarmRecord) item));
        reportAlarmRecordRepository.saveAll(save);
    }
}
